package gui;

import java.lang.*;
import java.util.*;
import java.io.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import gui.*;
import simulator.*;
 
public class GUI extends JApplet
{
    SimWinMgr simWinMgr;
    static SimulationParameters params = new SimulationParameters ();
    
    public void init ()
    {
	Container mainContainer = getContentPane();
	simWinMgr = new SimWinMgr ();
	mainContainer.add(simWinMgr, BorderLayout.CENTER);
	
	JMenuBar mainMenu = new MainMenu(simWinMgr);
	setJMenuBar(mainMenu);
	
	simWinMgr.createSimulationWindow (params);
    }
    
    public static void main (String[] args)
    {
	for (int i = 0; i < args.length; i++)
	    {
		String param = (String) args[i];
		
		try {
		    if (param.startsWith ("-")) {
			if (param.substring (0,5).equals ("-sim:")) {
			    params.setSimFile (param.substring (5));
			} else if (param.substring(0,5).equals ("-out:")) {
			    params.setOutFile (param.substring (5));
			} else if (param.substring (0,5).equals ("-err:")) {
			    params.setErrFile (param.substring (5));
			} else if (param.substring (0,6).equals ("-stat:")) {
			    params.setStatsFile (param.substring (6));
			} else if (param.substring (0,6).equals ("-batch")) {
			    params.setBatchMode (true);
			} else if (param.substring (0,7).equals ("-steps:")) {
			    int numSteps = new Integer (param.substring (7)).intValue ();
			    params.setMaxSteps (numSteps);
			} else if (param.substring (0,9).equals ("-program:")) {
			    params.addProgramFile (param.substring (9));
			} else {
			    System.err.println ("Unrecognized option: " + param);
			}
		    } else {
			// Its a file name
			int extchar = param.indexOf ('.');

			if (extchar == -1) {
			    System.err.println ("Unrecognized file: " + param);
			} else {
			    String extension = param.substring (extchar);

			    if (extension.equals (".cpa")) {
				params.addProgramFile (param);
			    } else if (extension.equals (".sim")) {
				params.setSimFile (param);
			    } else {
				System.err.println ("Unrecognized file extension: " + extension);
			    }
			}
		    }
		} catch(StringIndexOutOfBoundsException e) {
		    System.err.println (e.toString ());
		    System.err.println (param);
		}
	    }
	
	MainWindow mainWindow = new MainWindow ();
	mainWindow.setSize(750,750);
	mainWindow.setVisible(true);
	
	if (params.parametersDefined ()) {
	    mainWindow.startSimulation (params);
	} else {
	    System.err.println("Usage: java gui.GUI [-batch] <simFile>"+
			       " <cellProgramFile>* [-stats:<statFile>]"+
			       " [-out:<outFile>] [-err:<errFile>]");
	    System.exit(0);
	}

	/* old batch mode code no longer supported, everything in parameters
		if (simFile.equals("") && cellProgramFiles.length () == 0)
		    {
			System.err.println("Usage: java gui.GUI [-batch] -sim:<simFile>"+
					   " -program:<cellProgramFile> [-stats:<statFile>]"+
					   " [-out:<outFile>] [-err:<errFile>]");
			System.exit(0);
		    }
		
		BufferedReader cellProgramFileStream;
		BufferedReader simFileStream;
		BufferedWriter outFileStream;
		BufferedWriter statsFileStream;
		BufferedWriter errFileStream;
		Vector ioObjects = new Vector();
		
		try
		    {
			simFileStream = new BufferedReader(new FileReader(simFile));
			cellProgramFileStream = new BufferedReader(new FileReader(cellProgramFile));
			if(outFile.equals(""))
			    {
				System.out.println("OUT = System.out");
				outFileStream = new BufferedWriter(new PrintWriter(System.out));
			    }
			else
			    {
				outFileStream = new BufferedWriter(new FileWriter(outFile));
			    }
			if(statsFile.equals(""))
			    {
				System.out.println("STAT = System.out");
				statsFileStream = new BufferedWriter(new PrintWriter(System.out));
			    }
			else
			    {
				statsFileStream = new BufferedWriter(new FileWriter(statsFile));
			    }
			if(errFile.equals(""))
			    {
				System.out.println("ERR = System.err");
				errFileStream = new BufferedWriter(new PrintWriter(System.err));
			    }
			else
			    {
				errFileStream = new BufferedWriter(new FileWriter(errFile));
			    }
			ioObjects.add(simFileStream);
			ioObjects.add(cellProgramFileStream);
			ioObjects.add(outFileStream);
			ioObjects.add(statsFileStream);
			ioObjects.add(errFileStream);
			
			Simulation theSimulation = new Simulation (ioObjects);
			for (int i=0; i<numSteps; i++) {
			    theSimulation.nextStep();
			}
			
			theSimulation.finish();
		    }
		catch(FileNotFoundException e)
		    {
			System.err.println("Unable to open file: "+e.toString());
			return;
		    }
		catch(IOException e)
		    {
			System.err.println("Unable to open file: "+e.toString());
			return;
			}
	*/
    }
}
