package gui;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import gui.*;
import simulator.*;

class SimWinMgr extends JDesktopPane
{
    Hashtable simWindows;
    Integer curSimWin;
    Integer nextWindowID;
    
    Vector savedOptionsFileNew = null;
    
    public SimWinMgr ()
    {
	super ();
	setOpaque (false);
	
	simWindows = new Hashtable ();
	curSimWin = null;
	nextWindowID = new Integer (1);
    }
    
    public Vector getSavedOptionsFileNew ()
    {
	return savedOptionsFileNew;
    }
    
    public void setSavedOptionsFileNew (Vector pSave)
    {
	savedOptionsFileNew = pSave;
    }
    
    void createSimulationWindow (SimulationParameters params)
    {
	Rectangle rect = getBounds ();
	int width = (int) rect.getWidth ();
	int height = (int) rect.getHeight();
	int x = ((nextWindowID.intValue()-1)%5)*width/3/4;
	int y = ((nextWindowID.intValue()-1)%5)*height/3/4;
	width = (int)(width*2.0/3.0);
	height = (int)(height*2.0/3.0);
	rect.setLocation(x,y);
	rect.setSize(width,height);
	
	SimulationWindow simWin = new SimulationWindow (nextWindowID, "Simulation", rect, this, params);
	add(simWin, JLayeredPane.DEFAULT_LAYER);
	simWin.setVisible(true);
	
	simWindows.put(new Integer(nextWindowID.intValue()), simWin);
	nextWindowID = new Integer(nextWindowID.intValue() + 1);
    }
    
    void setCurrentSimulationWindow(Integer windowID)
    {
	curSimWin = new Integer(windowID.intValue());
    }
    
    void closeCurrentSimulationWindow ()
    {
	if (curSimWin != null) {
	    SimulationWindow simWin = (SimulationWindow)simWindows.get (curSimWin);
	    curSimWin = null;

	    if (simWin != null) {
		try {
		    simWin.setClosed(true);
		} catch (java.beans.PropertyVetoException e) {
		    ;
		}
	    }
	}
    }
    
    void deleteSimulationWindow(Integer windowID)
    {
	simWindows.remove(new Integer(windowID.intValue()));
    }
    
    void nextStep()
    {
	if (curSimWin != null) {
	    SimulationWindow simWin = (SimulationWindow)simWindows.get(curSimWin);
	    if (simWin != null)
		{
		    simWin.nextStep();
		}
	}
    }
    
    void gotoStep(int stepNumber)
    {
	if (curSimWin != null)
	    {
		SimulationWindow simWin = (SimulationWindow)simWindows.get(curSimWin);
		if (simWin != null) {
		    simWin.gotoStep(stepNumber);
		}
	    }
    }
    
    boolean exit ()
    {
	int retVal;
	retVal = JOptionPane.showConfirmDialog(getParent(), "Are you sure?", "Exit CellSim", JOptionPane.YES_NO_OPTION);
	if(retVal == JOptionPane.YES_OPTION)
	    {
		return true;
	    }
	else
	    {
		return false;
	    }
    }
}
