package sysmodels.simplebiology;

import java.util.*;
import simulator.*;
import util.*;

class CellDetails
{
    Cell cell;
    int x, y, z;
    Vector sensedChemicalsXplus;
    Vector sensedChemicalsXminus;
    Vector sensedChemicalsYplus;
    Vector sensedChemicalsYminus;
    Vector sensedChemicalsZplus;
    Vector sensedChemicalsZminus;
    Vector sensedChemicals;
    
    boolean isNewCell;
    
    CellDetails(Cell p_cell)
    {
	cell = p_cell;
	sensedChemicals = new Vector();
	sensedChemicalsXplus = new Vector();
	sensedChemicalsXminus = new Vector();
	sensedChemicalsYplus = new Vector();
	sensedChemicalsYminus = new Vector();
	sensedChemicalsZplus = new Vector();
	sensedChemicalsZminus = new Vector();
	isNewCell = true;
    }
    
    public Cell getCell() { return cell; }
    public int getX() {	return x; }
    public int getY() {	return y; }
    public int getZ() {	return z; }
    public void setPosition(Position p_pos) { x = (int)p_pos.getX(); y = (int)p_pos.getY();	z = (int)p_pos.getZ(); }
    public void setPosition(int p_x, int p_y, int p_z) { x = p_x; y = p_y; z = p_z; }
    public void setPosition(double p_x, double p_y, double p_z)	{ x = (int)p_x;	y = (int)p_y; z = (int)p_z; }
    public Vector getSensedChemicals() { return sensedChemicals; }
    public boolean isNew() { return isNewCell; }
    public void setAsNew() { isNewCell = true; }
    public void setAsOld() { isNewCell = false;}
    
    public void diffuseChemicals(CellDetails cd, Vector emissions)
    {
	Vector sensor;
	
	Enumeration enum = emissions.elements();
	while(enum.hasMoreElements())
	    {
		CellEmission emission = (CellEmission)enum.nextElement();
		if(emission.getDirection().equals("all"))
		    {
			if((x > cd.getX()))	addChemicalIntoSensorBin(getSensor("X-"),emission);
			if((x < cd.getX()))	addChemicalIntoSensorBin(getSensor("X+"),emission);
			if((y > cd.getY()))	addChemicalIntoSensorBin(getSensor("Y-"),emission);
			if((y < cd.getY()))	addChemicalIntoSensorBin(getSensor("Y+"),emission);
			if((z > cd.getZ()))	addChemicalIntoSensorBin(getSensor("Z-"),emission);
			if((z < cd.getZ()))	addChemicalIntoSensorBin(getSensor("Z+"),emission);
			addChemicalIntoMaxSensed(emission);
		    }
		else
		    {
			sensor = getSensor(emission.getDirection());
			addChemicalIntoSensorBin(sensor,emission);
			addChemicalIntoMaxSensed(emission);
		    }
	    }
    }
    
    public void addSensedChemical(CellEmission emission)
    {
	addChemicalIntoMaxSensed(emission);
    }
    
    private void addChemicalIntoSensorBin(Vector sensor, CellEmission emission)
    {
	boolean chemWasFound = false;
	
	if(sensor == null)
	    return;
	
	Enumeration enum = sensor.elements();
	while(enum.hasMoreElements())
	    {
		ChemicalConcentration cc = (ChemicalConcentration) enum.nextElement();
		if(emission.getChemical().equals(cc.getChemical()))
		    {
			cc.setConcentration(cc.getConcentration()+emission.getConcentration());
			chemWasFound = true;
			break;
		    }
	    }
	
	if(!chemWasFound)
	    {
		sensor.add(new ChemicalConcentration(emission.getChemical(),emission.getConcentration()));
	    }		
    }
    
    private void addChemicalIntoMaxSensed(CellEmission emission)
    {
	boolean chemWasFound = false;
	
	Enumeration enum = sensedChemicals.elements();
	while(enum.hasMoreElements())
	    {
		ChemicalConcentration cc = (ChemicalConcentration) enum.nextElement();
		if(emission.getChemical().equals(cc.getChemical()))
		    {
			if(cc.getConcentration() < emission.getConcentration())
			    cc.setConcentration(emission.getConcentration());
			chemWasFound = true;
			break;
		    }
	    }
	
	if(!chemWasFound)
	    {
		sensedChemicals.add(new ChemicalConcentration(emission.getChemical(),emission.getConcentration()));
	    }		
    }
    
    public void disposeSensedChemicals()
    {
	// Can't make these clear's instead --- the old vector's are used somewhere?
	sensedChemicals = new Vector();
	sensedChemicalsXplus = new Vector();
	sensedChemicalsXminus = new Vector();
	sensedChemicalsYplus = new Vector();
	sensedChemicalsYminus = new Vector();
	sensedChemicalsZplus = new Vector();
	sensedChemicalsZminus = new Vector();
    }
    
    public Vector getSensor(String dir)
    {
	if(dir.equals("X+")) return sensedChemicalsXplus;
	if(dir.equals("X-")) return sensedChemicalsXminus;
	if(dir.equals("Y+")) return sensedChemicalsYplus;
	if(dir.equals("Y-")) return sensedChemicalsYminus;
	if(dir.equals("Z+")) return sensedChemicalsZplus;
	if(dir.equals("Z-")) return sensedChemicalsZminus;
	return null;
    }
    
    public String toString()
    {
	StringBuffer sb = new StringBuffer();
	String[] A = new String[7];
	A[0] = "X+"; A[1] = "X-"; A[2] = "Y+"; A[3] = "Y-"; A[4] = "Z+"; A[5] = "Z-"; A[6] = "AL";
	
	sb.append(cell.getID() + " : X = " + x + " : Y = " + y + " : Z = " + z + "\n");
	for(int i=0; i<7; i++)
	    {
		Vector sensor = getSensor(A[i]);
		if(sensor == null) sensor = sensedChemicals;
		Enumeration enum = sensor.elements();
		sb.append(" | " + A[i] + " ");
		while(enum.hasMoreElements())
		    {
			ChemicalConcentration cc = (ChemicalConcentration)enum.nextElement();
			sb.append(cc.toString() + ",");
		    }
	    }
	sb.append("\n");
	return sb.toString();
    }
}
